/*
 * Copyright (c) 2003 Marcus Glocker
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <sys/types.h>
#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* prototypes */

void	usage(void);
void	help(void);
int	makeweek(const int, int *);

int	month[13] = {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

const char	sdayw_e[7][128] = {
	"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday",
	"Saturday"};
const char	sdayw_g[7][128] = {
	"Sonntag", "Montag", "Dienstag", "Mittwoch", "Donnerstag", "Freitag",
	"Samstag"};

const char	smonth_e[13][128] = {
	"", "January", "February", "March", "Avril", "May", "June", "July",
	"August", "September", "October", "November", "December"};

const char	smonth_g[13][128] = {
	"", "Januar", "Februar", "Maerz", "April", "Mai", "Juni", "Juli",
	"August", "September", "Oktober", "November", "Dezember"};

void
usage()
{
	extern char	*__progname;

	fprintf(stderr, "usage: %s ", __progname);
	fprintf(stderr, "[-w weekday] [-l language]\n");

	exit(1);
}

int
main(int argc, char *argv[])
{
	int		ch, optlang = 0;
	int		nextday, nextmonth, nextyear, nextdayw = 0;
	int		nowday, nowmonth, nowyear, nowdayw;
	int		adday[7];
	time_t		tnow;
	struct tm	*lt;

	if (argc < 2)
		usage();

	while ((ch = getopt(argc, argv, "w:l:")) != -1) {
		switch (ch) {
			case 'w':
				nextdayw = atoi(optarg);
				if (nextdayw >= 0 && nextdayw <= 6)
					break;
				else
					usage();
			case 'l':
				if (strcmp(optarg, "german") == 0)
					optlang = 1;
				else
					optlang = 0;
				break;
			default:
				usage();
				break;
		}
	}
	if (optind == 1)
		usage();

	/* fill adday for given weekday */
	makeweek(nextdayw, adday);

	/* receive now date */
	time(&tnow);
	lt = localtime(&tnow);

	/* fill now date to new vars */
	nowday = lt->tm_mday;
	nowmonth = lt->tm_mon + 1;
	nowyear = lt->tm_year + 1900;
	nowdayw = lt->tm_wday;

	/* check for leap year */
	if (nowmonth == 2 && nowyear % 4 == 0)
		month[nowmonth] = 29;

	/* calculate next weekday date */
	nextday = nowday + adday[nowdayw];
	nextmonth = nowmonth;
	nextyear = nowyear;

	/* correct calculated if necessary */
	if (nextday > month[nowmonth]) {
		nextday = nextday - month[nowmonth];
		nextmonth = nowmonth + 1;
		if (nextmonth > 12) {
			nextmonth = 1;
			nextyear = nowyear + 1;
		}
	}

	/* print result */
	if (optlang == 0) {
		fprintf(stdout, "%s, %d. %s %d\n",
		    sdayw_e[nextdayw], nextday, smonth_e[nextmonth], nextyear);
	} else {
		fprintf(stdout, "%s, %d. %s %d\n",
		    sdayw_g[nextdayw], nextday, smonth_g[nextmonth], nextyear);
	}

	return 0;
}

int
makeweek(int weekday, int *week)
{
	int	i;

	for (i = 6; i >= 0; i--) {
		if (weekday == 6)
			weekday = 0;
		else
			weekday++;

		week[weekday] = i;
	}	

	return 0;
}
